@@ -11,7 +11,7 @@
 #include "Profile.h"
 
 // Qt
-#include <QRegularExpression>
+#include <QFileInfo>
 #include <QTextCodec>
 
 // KDE
@@ -77,6 +77,10 @@ const std::vector<Profile::PropertyInfo> Profile::DefaultProperties = {
     {TerminalRows, "TerminalRows", GENERAL_GROUP, 28},
     {TerminalMargin, "TerminalMargin", GENERAL_GROUP, 1},
     {TerminalCenter, "TerminalCenter", GENERAL_GROUP, false},
+    {ErrorBars, "ErrorBars", GENERAL_GROUP, 2},
+    {ErrorBackground, "ErrorBackground", GENERAL_GROUP, 1},
+    {AlternatingBars, "AlternatingBars", GENERAL_GROUP, 2},
+    {AlternatingBackground, "AlternatingBackground", GENERAL_GROUP, 1},
 
     // Appearance
     {Font, "Font", APPEARANCE_GROUP, QFont()},
@@ -90,6 +94,11 @@ const std::vector<Profile::PropertyInfo> Profile::DefaultProperties = {
     {DimValue, "DimmValue", APPEARANCE_GROUP, 128},
     {DimWhenInactive, "DimWhenInactive", GENERAL_GROUP, false},
     {InvertSelectionColors, "InvertSelectionColors", GENERAL_GROUP, false},
+    {EmojiFont, "EmojiFont", APPEARANCE_GROUP, QFont()},
+    {WordMode, "WordMode", APPEARANCE_GROUP, false},
+    {WordModeAttr, "WordModeAttr", APPEARANCE_GROUP, true},
+    {WordModeAscii, "WordModeAscii", APPEARANCE_GROUP, true},
+    {WordModeBrahmic, "WordModeBrahmic", APPEARANCE_GROUP, true},
 
 // Keyboard
 #ifdef Q_OS_MACOS
@@ -112,11 +121,14 @@ const std::vector<Profile::PropertyInfo> Profile::DefaultProperties = {
     {BlinkingTextEnabled, "BlinkingTextEnabled", TERMINAL_GROUP, true},
     {FlowControlEnabled, "FlowControlEnabled", TERMINAL_GROUP, true},
     {BidiRenderingEnabled, "BidiRenderingEnabled", TERMINAL_GROUP, true},
+    {BidiLineLTR, "BidiLineLTR", TERMINAL_GROUP, true},
+    {BidiTableDirOverride, "BidiTableDirOverride", TERMINAL_GROUP, true},
     {BlinkingCursorEnabled, "BlinkingCursorEnabled", TERMINAL_GROUP, false},
     {BellMode, "BellMode", TERMINAL_GROUP, Enum::NotifyBell},
     {VerticalLine, "VerticalLine", TERMINAL_GROUP, false},
     {VerticalLineAtChar, "VerticalLineAtChar", TERMINAL_GROUP, 80},
     {PeekPrimaryKeySequence, "PeekPrimaryKeySequence", TERMINAL_GROUP, QString()},
+    {LineNumbers, "LineNumbers", TERMINAL_GROUP, 0},
 
     // Cursor
     {UseCustomCursorColor, "UseCustomCursorColor", CURSOR_GROUP, false},
@@ -180,7 +192,10 @@ static QString defaultShell()
         proc.setArguments({QStringLiteral("passwd"), QString::number(pw->pw_uid)});
         KSandbox::startHostProcess(proc);
         proc.waitForFinished();
-        return QString::fromUtf8(proc.readAllStandardOutput().simplified().split(':').at(6));
+        const auto output = proc.readAllStandardOutput().simplified();
+        if (auto parts = QString::fromUtf8(output).split(QLatin1Char(':')); output.size() >= 7) {
+            return parts.at(6);
+        }
     }
     return {};
 #endif
@@ -220,7 +235,7 @@ void Profile::useBuiltin()
 }
 
 Profile::Profile(const Profile::Ptr &parent)
-    : _propertyValues(QHash<Property, QVariant>())
+    : _propertyValues(PropertyMap())
     , _parent(parent)
     , _hidden(false)
 {
@@ -247,6 +262,20 @@ bool Profile::isBuiltin() const
     return path() == BUILTIN_MAGIC_PATH;
 }
 
+bool Profile::isEditable() const
+{
+    // Read-only profiles (i.e. with non-user-writable .profile location)
+    // aren't editable. This includes the built-in profile, which is hardcoded.
+    return !isBuiltin() && QFileInfo(path()).isWritable();
+}
+
+bool Profile::isDeletable() const
+{
+    // To delete a file, parent dir must be writable
+    const QFileInfo fileInfo(path());
+    return !isBuiltin() && fileInfo.exists() && QFileInfo(fileInfo.path()).isWritable();
+}
+
 bool Profile::isHidden() const
 {
     return _hidden;
@@ -267,22 +296,42 @@ const Profile::Ptr Profile::parent() const
 
 bool Profile::isEmpty() const
 {
-    return _propertyValues.isEmpty();
+    return _propertyValues.empty();
 }
 
-QHash<Profile::Property, QVariant> Profile::setProperties() const
+const Profile::PropertyMap &Profile::properties() const
 {
     return _propertyValues;
 }
 
 void Profile::setProperty(Property p, const QVariant &value)
 {
-    _propertyValues.insert(p, value);
+    _propertyValues.insert_or_assign(p, value);
+}
+
+void Profile::setProperty(Property p, QVariant &&value)
+{
+    _propertyValues.insert_or_assign(p, value);
+}
+
+void Profile::assignProperties(const PropertyMap &map)
+{
+    for (const auto &[p, value] : map) {
+        setProperty(p, value);
+    }
+}
+
+void Profile::assignProperties(PropertyMap &&map)
+{
+    // If a key exists in both maps, we want to use the associated
+    // value from 'map'
+    map.merge(_propertyValues);
+    _propertyValues.swap(map);
 }
 
 bool Profile::isPropertySet(Property p) const
 {
-    return _propertyValues.contains(p);
+    return _propertyValues.find(p) != _propertyValues.cend();
 }
 
 Profile::Property Profile::lookupByName(const QString &name)
