@@ -506,12 +506,12 @@ void Session::run()
     addEnvironmentEntry(QStringLiteral("KONSOLE_DBUS_SESSION=%1").arg(dbusObject));
 
 #if KCOREADDONS_VERSION >= QT_VERSION_CHECK(5, 97, 0)
+    const auto originalEnvironment = _shellProcess->environment();
     _shellProcess->setProgram(exec);
     _shellProcess->setArguments(arguments);
-    _shellProcess->setEnvironment(_environment);
+    _shellProcess->setEnvironment(originalEnvironment + _environment);
     const auto context = KSandbox::makeHostContext(*_shellProcess);
-    // The Pty class is incredibly janky and will topple over when starting with environment, so unset it again.
-    _shellProcess->setEnvironment({});
+    _shellProcess->setEnvironment(originalEnvironment);
     const auto result = _shellProcess->start(context.program, context.arguments, _environment);
 #else
     int result = _shellProcess->start(exec, arguments, _environment);
@@ -1169,48 +1169,102 @@ QString Session::getDynamicTitle()
     QString title = tabTitleFormat(Session::LocalTabTitle);
     // search for and replace known marker
 
-    int UID = process->userId(&ok);
-    if (!ok) {
-        title.replace(QLatin1String("%B"), QStringLiteral("-"));
-    } else {
-        // title.replace(QLatin1String("%I"), QString::number(UID));
-        if (UID == 0) {
-            title.replace(QLatin1String("%B"), QStringLiteral("#"));
-        } else {
-            title.replace(QLatin1String("%B"), QStringLiteral("$"));
-        }
-    }
-
-    title.replace(QLatin1String("%u"), process->userName());
-    title.replace(QLatin1String("%h"), Konsole::ProcessInfo::localHost());
-    title.replace(QLatin1String("%n"), process->name(&ok));
-
-    title.replace(QLatin1String("%w"), userTitle());
-    title.replace(QLatin1String("%#"), QString::number(sessionId()));
-
     QString dir = _reportedWorkingUrl.toLocalFile();
-    ok = true;
+    bool dirOk = true;
     if (dir.isEmpty()) {
         // update current directory from process
         updateWorkingDirectory();
         // Previous process may have been freed in updateSessionProcessInfo()
         process = getProcessInfo();
-        dir = process->currentDir(&ok);
+        dir = process->currentDir(&dirOk);
     }
-    if (!ok) {
-        title.replace(QLatin1String("%d"), QStringLiteral("-"));
-        title.replace(QLatin1String("%D"), QStringLiteral("-"));
-    } else {
-        // allow for shortname to have the ~ as homeDir
-        const QString homeDir = process->userHomeDir();
-        if (!homeDir.isEmpty()) {
-            if (dir.startsWith(homeDir)) {
-                dir.remove(0, homeDir.length());
-                dir.prepend(QLatin1Char('~'));
+
+    int pos = 0;
+    while ((pos = title.indexOf(QLatin1Char('%'), pos)) != -1) {
+        if (pos >= title.size() - 1) {
+            break;
+        }
+
+        switch (title.at(pos + 1).toLatin1()) {
+        case 'B': {
+            int UID = process->userId(&ok);
+            if (!ok) {
+                title.replace(pos, 2, QStringLiteral("-"));
+                pos++;
+            } else {
+                // title.replace(QLatin1String("%I"), QString::number(UID));
+                if (UID == 0) {
+                    title.replace(pos, 2, QStringLiteral("#"));
+                    pos++;
+                } else {
+                    title.replace(pos, 2, QStringLiteral("$"));
+                    pos++;
+                }
             }
+        } break;
+        case 'u': {
+            QString replacement = process->userName();
+            title.replace(pos, 2, replacement);
+            pos += replacement.size();
+        } break;
+        case 'h': {
+            QString replacement = Konsole::ProcessInfo::localHost();
+            title.replace(pos, 2, replacement);
+            pos += replacement.size();
+        } break;
+        case 'n': {
+            QString replacement = process->name(&ok);
+            title.replace(pos, 2, replacement);
+            pos += replacement.size();
+        } break;
+        case 'w': {
+            QString replacement = userTitle();
+            title.replace(pos, 2, replacement);
+            pos += replacement.size();
+        } break;
+        case '#': {
+            QString replacement = QString::number(sessionId());
+            title.replace(pos, 2, replacement);
+            pos += replacement.size();
+        } break;
+        case 'd':
+            if (!dirOk) {
+                title.replace(pos, 2, QStringLiteral("-"));
+                pos++;
+            } else {
+                // allow for shortname to have the ~ as homeDir
+                const QString homeDir = process->userHomeDir();
+                if (!homeDir.isEmpty()) {
+                    if (dir.startsWith(homeDir)) {
+                        dir.remove(0, homeDir.length());
+                        dir.prepend(QLatin1Char('~'));
+                    }
+                }
+                const QString replacement = process->formatShortDir(dir);
+                title.replace(pos, 2, replacement);
+                pos += replacement.size();
+            }
+            break;
+        case 'D':
+            if (!dirOk) {
+                title.replace(pos, 2, QStringLiteral("-"));
+                pos++;
+            } else {
+                // allow for shortname to have the ~ as homeDir
+                const QString homeDir = process->userHomeDir();
+                if (!homeDir.isEmpty()) {
+                    if (dir.startsWith(homeDir)) {
+                        dir.remove(0, homeDir.length());
+                        dir.prepend(QLatin1Char('~'));
+                    }
+                }
+                title.replace(pos, 2, dir);
+                pos += dir.size();
+            }
+            break;
+        default:
+            pos++;
         }
-        title.replace(QLatin1String("%D"), dir);
-        title.replace(QLatin1String("%d"), process->formatShortDir(dir));
     }
 
     return title;
@@ -1808,6 +1862,17 @@ void Session::setReadOnly(bool readOnly)
         Q_EMIT readOnlyChanged();
     }
 }
+bool Session::getSelectMode() const
+{
+    return _selectMode;
+}
+
+void Session::setSelectMode(bool mode)
+{
+    if (_selectMode != mode) {
+        _selectMode = mode;
+    }
+}
 
 void Session::setColor(const QColor &color)
 {
