@@ -22,7 +22,6 @@
 #include <QKeyEvent>
 #include <QList>
 #include <QMenu>
-#include <QPainter>
 #include <QStandardPaths>
 #include <QUrl>
 
@@ -46,7 +45,12 @@
 
 #include <KIO/CommandLauncherJob>
 
+#include <kio_version.h>
+#if KIO_VERSION >= QT_VERSION_CHECK(5, 98, 0)
+#include <KIO/JobUiDelegateFactory>
+#else
 #include <KIO/JobUiDelegate>
+#endif
 #include <KIO/OpenFileManagerWindowJob>
 #include <KIO/OpenUrlJob>
 
@@ -435,7 +439,7 @@ void SessionController::openUrl(const QUrl &url)
     } else {
         // TODO Implement handling for other Url types
 
-        KMessageBox::sorry(view()->window(), i18n("Konsole does not know how to open the bookmark: ") + url.toDisplayString());
+        KMessageBox::error(view()->window(), i18n("Konsole does not know how to open the bookmark: ") + url.toDisplayString());
 
         qCDebug(KonsoleDebug) << "Unable to open bookmark at url" << url << ", I do not know"
                               << " how to handle the protocol " << url.scheme();
@@ -448,12 +452,14 @@ void SessionController::setupPrimaryScreenSpecificActions(bool use)
     QAction *clearAction = collection->action(QStringLiteral("clear-history"));
     QAction *resetAction = collection->action(QStringLiteral("clear-history-and-reset"));
     QAction *selectAllAction = collection->action(QStringLiteral("select-all"));
+    QAction *selectModeAction = collection->action(QStringLiteral("select-mode"));
     QAction *selectLineAction = collection->action(QStringLiteral("select-line"));
 
     // these actions are meaningful only when primary screen is used.
     clearAction->setEnabled(use);
     resetAction->setEnabled(use);
     selectAllAction->setEnabled(use);
+    selectModeAction->setEnabled(use);
     selectLineAction->setEnabled(use);
 }
 
@@ -552,7 +558,11 @@ void SessionController::handleWebShortcutAction(QAction *action)
         const QUrl url = filterData.uri();
 
         auto *job = new KIO::OpenUrlJob(url);
+#if KIO_VERSION >= QT_VERSION_CHECK(5, 98, 0)
+        job->setUiDelegate(KIO::createDefaultJobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, QApplication::activeWindow()));
+#else
         job->setUiDelegate(new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, QApplication::activeWindow()));
+#endif
         job->start();
     }
 }
@@ -708,6 +718,11 @@ void SessionController::setupCommonActions()
     action->setText(i18n("&Select All"));
     action->setIcon(QIcon::fromTheme(QStringLiteral("edit-select-all")));
 
+    action = collection->addAction(QStringLiteral("select-mode"), this, &SessionController::selectMode);
+    action->setText(i18n("Select &Mode"));
+    action->setIcon(QIcon::fromTheme(QStringLiteral("edit-select")));
+    collection->setDefaultShortcut(action, QKeySequence(Qt::CTRL | Qt::SHIFT | Qt::Key_D));
+
     action = collection->addAction(QStringLiteral("select-line"), this, &SessionController::selectLine);
     action->setText(i18n("Select &Line"));
 
@@ -938,7 +953,7 @@ void SessionController::switchProfile(const Profile::Ptr &profile)
 void SessionController::setEditProfileActionText(const Profile::Ptr &profile)
 {
     QAction *action = actionCollection()->action(QStringLiteral("edit-current-profile"));
-    if (profile->isBuiltin()) {
+    if (!profile->isEditable()) {
         action->setText(i18n("Create New Profile..."));
     } else {
         action->setText(i18n("Edit Current Profile..."));
@@ -973,8 +988,8 @@ void SessionController::editCurrentProfile()
 
     auto profile = SessionManager::instance()->sessionProfile(session());
     auto state = EditProfileDialog::ExistingProfile;
-    // Don't edit the built-in profile, instead create a new one
-    if (profile->isBuiltin()) {
+    // Don't edit uneditable profiles, instead create a new one
+    if (!profile->isEditable()) {
         auto newProfile = Profile::Ptr(new Profile(profile));
         newProfile->clone(profile, true);
         const QString uniqueName = ProfileManager::instance()->generateUniqueName();
@@ -1067,13 +1082,21 @@ bool SessionController::confirmClose() const
                 title);
         }
 
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+        int result = KMessageBox::warningTwoActions(view()->window(),
+#else
         int result = KMessageBox::warningYesNo(view()->window(),
-                                               question,
-                                               i18n("Confirm Close"),
-                                               KGuiItem(i18nc("@action:button", "Close Program"), QStringLiteral("application-exit")),
-                                               KStandardGuiItem::cancel(),
-                                               QStringLiteral("CloseSingleTab"));
+#endif
+                                                    question,
+                                                    i18n("Confirm Close"),
+                                                    KGuiItem(i18nc("@action:button", "Close Program"), QStringLiteral("application-exit")),
+                                                    KStandardGuiItem::cancel(),
+                                                    QStringLiteral("CloseSingleTab"));
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+        return result == KMessageBox::PrimaryAction;
+#else
         return result == KMessageBox::Yes;
+#endif
     }
     return true;
 }
@@ -1102,12 +1125,20 @@ bool SessionController::confirmForceClose() const
                 title);
         }
 
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+        int result = KMessageBox::warningTwoActions(view()->window(),
+#else
         int result = KMessageBox::warningYesNo(view()->window(),
-                                               question,
-                                               i18n("Confirm Close"),
-                                               KGuiItem(i18nc("@action:button", "Kill Program"), QStringLiteral("application-exit")),
-                                               KStandardGuiItem::cancel());
+#endif
+                                                    question,
+                                                    i18n("Confirm Close"),
+                                                    KGuiItem(i18nc("@action:button", "Kill Program"), QStringLiteral("application-exit")),
+                                                    KStandardGuiItem::cancel());
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+        return result == KMessageBox::PrimaryAction;
+#else
         return result == KMessageBox::Yes;
+#endif
     }
     return true;
 }
@@ -1159,7 +1190,11 @@ void SessionController::openBrowser()
     } else {
         const QUrl currentUrl = url().isLocalFile() ? url() : QUrl::fromLocalFile(QDir::homePath());
         auto *job = new KIO::OpenUrlJob(currentUrl);
+#if KIO_VERSION >= QT_VERSION_CHECK(5, 98, 0)
+        job->setUiDelegate(KIO::createDefaultJobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, QApplication::activeWindow()));
+#else
         job->setUiDelegate(new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, QApplication::activeWindow()));
+#endif
         job->start();
     }
 }
@@ -1196,6 +1231,17 @@ void SessionController::selectAll()
 {
     view()->selectAll();
 }
+void SessionController::selectMode()
+{
+    if (!session().isNull()) {
+        QAction *readonlyAction = actionCollection()->action(QStringLiteral("view-readonly"));
+        bool Mode = session()->getSelectMode();
+        session()->setSelectMode(!Mode);
+        readonlyAction->setEnabled(Mode);
+        view()->setSelectMode(!Mode);
+    }
+}
+
 void SessionController::selectLine()
 {
     view()->selectCurrentLine();
@@ -1551,7 +1597,7 @@ void SessionController::searchTextChanged(const QString &text)
     _searchText = text;
 
     if (text.isEmpty()) {
-        view()->screenWindow()->clearSelection();
+        view()->clearMouseSelection();
         view()->screenWindow()->scrollTo(_searchStartLine);
     }
 
@@ -1661,7 +1707,7 @@ void SessionController::changeSearchMatch()
     Q_ASSERT(_searchFilter);
 
     // reset Selection for new case match
-    view()->screenWindow()->clearSelection();
+    view()->clearMouseSelection();
     beginSearch(_searchBar->searchText(), reverseSearchChecked() ? Enum::BackwardsSearch : Enum::ForwardsSearch);
 }
 void SessionController::showHistoryOptions()
@@ -1909,7 +1955,7 @@ void SessionController::showDisplayContextMenu(const QPoint &position)
         for (auto *elm : old) {
             newActions.removeAll(elm);
         }
-        // Finish Ading the "Open Folder With" action.
+        // Finish Adding the "Open Folder With" action.
 
         QList<QAction *> toRemove;
         // prepend content-specific actions such as "Open Link", "Copy Email Address" etc
@@ -2016,7 +2062,7 @@ void SessionController::zmodemDownload()
 void SessionController::zmodemUpload()
 {
     if (session()->isZModemBusy()) {
-        KMessageBox::sorry(view(), i18n("<p>The current session already has a ZModem file transfer in progress.</p>"));
+        KMessageBox::information(view(), i18n("<p>The current session already has a ZModem file transfer in progress.</p>"));
         return;
     }
 
@@ -2025,7 +2071,7 @@ void SessionController::zmodemUpload()
         zmodem = QStandardPaths::findExecutable(QStringLiteral("lsz"));
     }
     if (zmodem.isEmpty()) {
-        KMessageBox::sorry(view(),
+        KMessageBox::error(view(),
                            i18n("<p>No suitable ZModem software was found on this system.</p>"
                                 "<p>You may wish to install the 'rzsz' or 'lrzsz' package.</p>"));
         return;
