@@ -254,11 +254,12 @@ void TabbedViewContainer::moveActiveView(MoveDirection direction)
 
 void TabbedViewContainer::terminalDisplayDropped(TerminalDisplay *terminalDisplay)
 {
-    if (terminalDisplay->sessionController()->parent() != connectedViewManager()) {
+    auto *controller = terminalDisplay->sessionController();
+    if (controller->parent() != connectedViewManager()) {
         // Terminal from another window - recreate SessionController for current ViewManager
         disconnectTerminalDisplay(terminalDisplay);
-        Session *terminalSession = terminalDisplay->sessionController()->session();
-        Q_EMIT terminalDisplay->sessionController()->viewDragAndDropped();
+        Session *terminalSession = controller->session();
+        Q_EMIT controller->viewDragAndDropped(controller);
         connectedViewManager()->attachView(terminalDisplay, terminalSession);
         connectTerminalDisplay(terminalDisplay);
     }
@@ -330,6 +331,9 @@ void TabbedViewContainer::addView(TerminalDisplay *view)
     connect(viewSplitter, &ViewSplitter::destroyed, this, &TabbedViewContainer::viewDestroyed);
     connect(viewSplitter, &ViewSplitter::terminalDisplayDropped, this, &TabbedViewContainer::terminalDisplayDropped);
 
+    // Put this view on the foreground if it requests so, eg. on bell activity
+    connect(view, &TerminalDisplay::activationRequest, this, &Konsole::TabbedViewContainer::activateView);
+
     setCurrentIndex(index);
     Q_EMIT viewAdded(view);
 }
@@ -340,6 +344,8 @@ void TabbedViewContainer::splitView(TerminalDisplay *view, Qt::Orientation orien
     viewSplitter->clearMaximized();
     viewSplitter->addTerminalDisplay(view, orientation);
     connectTerminalDisplay(view);
+    // Put this view on the foreground if it requests so, eg. on bell activity
+    connect(view, &TerminalDisplay::activationRequest, this, &Konsole::TabbedViewContainer::activateView);
 }
 
 void TabbedViewContainer::connectTerminalDisplay(TerminalDisplay *display)
@@ -388,6 +394,15 @@ void TabbedViewContainer::forgetView()
     }
 }
 
+void TabbedViewContainer::activateView(const QString & /*xdgActivationToken*/)
+{
+    if (QWidget *widget = qobject_cast<QWidget *>(sender())) {
+        auto topLevelSplitter = qobject_cast<ViewSplitter *>(widget->parentWidget())->getToplevelSplitter();
+        setCurrentWidget(topLevelSplitter);
+        widget->setFocus();
+    }
+}
+
 void TabbedViewContainer::activateNextView()
 {
     QWidget *active = currentWidget();
